'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = createAdaptor;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _redux = require('redux');

var _graphql = require('graphql');

var _utilsParseSchema = require('./utils/parseSchema');

var _utilsParseSchema2 = _interopRequireDefault(_utilsParseSchema);

var _utilsNormalize = require('./utils/normalize');

var _utilsNormalize2 = _interopRequireDefault(_utilsNormalize);

var _utilsMerge = require('./utils/merge');

var _utilsMerge2 = _interopRequireDefault(_utilsMerge);

var UPDATE_CACHE = 'UPDATE_CACHE';

function createAdaptor(endpoint, schema) {
  var parsedSchema = (0, _utilsParseSchema2['default'])(schema);
  var reducers = Object.keys(parsedSchema).reduce(function (acc, key) {
    return _extends({}, acc, _defineProperty({}, key, createReducer(key)));
  });
  var reducer = (0, _redux.combineReducers)(reducers);
  var store = (0, _redux.createStore)(reducer);

  return {
    resolve: function resolve(queries, args, isDataLoaded) {
      var specs = queries(args);
      var query = Object.keys(specs).reduce(function (acc, key) {
        return acc + ' ' + specs[key];
      }, '');
      var graphQLQuery = ('\n        query AdrenalineQuery {\n          ' + query + '\n        }\n      ').replace(/\s+/g, ' ').trim();

      return (0, _graphql.graphql)(schema, graphQLQuery, store.getState()).then(function (res) {
        if (isDataLoaded(res.data)) {
          return res.data;
        }

        return request(endpoint, graphQLQuery).then(function (res) {
          store.dispatch({
            type: UPDATE_CACHE,
            payload: (0, _utilsNormalize2['default'])(parsedSchema, res.data)
          });

          return Promise.resolve();
        });
      });
    },

    subscribe: function subscribe(listener) {
      return store.subscribe(listener);
    },

    shouldComponentUpdate: function shouldComponentUpdate(prevState, nextState) {
      return prevState !== nextState;
    }
  };
}

function createReducer(key) {
  return function (state, _ref) {
    if (state === undefined) state = {};
    var type = _ref.type;
    var payload = _ref.payload;

    switch (type) {
      case UPDATE_CACHE:
        return (0, _utilsMerge2['default'])(state, payload[key] || {});
      default:
        return state;
    }
  };
}

function request(endpoint, query, variables, files) {
  if (!files) {
    return fetch(endpoint, {
      method: 'post',
      headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({ query: query })
    }).then(function (res) {
      return res.json();
    });
  }
}
module.exports = exports['default'];
//# sourceMappingURL=index.js.map